# Copyright 2018, 2019 Cumulus Networks, Inc.  All Rights reserved.
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt


import os
from netq_lib.orm import HOSTNAME
from netq_lib.orm.redisdb.models import ConfigMon
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate


def build_config_diff_objs(filename, filepath, current_modify_time, full_state_arr):

    new_config_mon_obj = ConfigMon()
    new_config_mon_obj.file_name = filename
    new_config_mon_obj.last_modify_time = current_modify_time

    with open(filepath, 'rb') as myfile:
        file_content = myfile.read().decode()

    new_config_mon_obj.file_content = file_content

    full_state_arr.append(new_config_mon_obj)



def process_config_mon(cmd_output, svc_failed=False, **kwargs):
    try:
        ##conf_dict = configuration.get_config("config-diff")
        conf_dict = {'config-diff': {'interfaces': '/etc/network/interfaces',
                                     'frr': '/etc/frr/frr.conf',
                                     'lldpd': '/etc/lldpd.d/README.conf',
                                     'ptm': '/etc/ptm.d/topology.dot'
                                     }
                     }
    except ValueError:
        m_configMon = ConfigMon.memmodel()
        for entry in m_configMon.query.filter(hostname=HOSTNAME):
            old_config_mon_ent = ConfigMon()
            old_config_mon_ent.copy(entry)
            old_config_mon_ent.delete()
        return ServiceStatus.FAIL

    fullstate = kwargs.get('fullstate')

    # grab current list of configMon objects
    m_configMon = ConfigMon.memmodel()
    old_configmon_objs = []
    new_configmon_objs = []
    full_state_arr = []
    deleteFileList = []
    if m_configMon:
        old_configmon_objs = (
            list(m_configMon.query.filter(hostname=HOSTNAME))
        )
    if conf_dict.get('config-diff'):
        for filename, filepath in list(conf_dict.get('config-diff').items()):
            if os.path.isfile(filepath):
                mem_modify_time = 0
                for old_entry in old_configmon_objs:
                    if old_entry.file_name == filename:
                        mem_modify_time = int(old_entry.last_modify_time)
                        break
                current_modify_time = os.path.getmtime(filepath)

                if fullstate:
                    build_config_diff_objs(filename, filepath, current_modify_time, full_state_arr)
                elif int(current_modify_time) > mem_modify_time:
                    new_config_mon_obj = ConfigMon()
                    new_config_mon_obj.file_name = filename
                    new_config_mon_obj.last_modify_time = current_modify_time

                    with open(filepath, 'rb') as myfile:
                        file_content = myfile.read().decode()

                    new_config_mon_obj.file_content = file_content

                    new_config_mon_obj.save()
                    new_configmon_objs.append(new_config_mon_obj)
            else:
                deleteFileList.append(filename)
    for to_be_deleted_file in deleteFileList:
        for old_obj in old_configmon_objs:
            if old_obj.file_name == to_be_deleted_file:
                old_configmon_ent = ConfigMon()
                old_configmon_ent.copy(old_obj)
                old_configmon_ent.delete()
                break


    if fullstate:
        save_fullstate(full_state_arr, ConfigMon().__class__.__name__)

    return ServiceStatus.OK
