# Copyright 2019 Cumulus Networks, Inc.  All Rights reserved.
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt


from netq_lib.orm import HOSTNAME
from netq_lib.orm.redisdb.models import RunningConfigMon
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate
import subprocess
import time

class ReturnGroup:
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

def runCommand(cmd):
    try:
        proc = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE)
        (out, err) = proc.communicate()
        if proc.returncode != 0:
            proc.terminate()
            time.sleep(0.3)
            proc.wait()
            return ReturnGroup(success=False, op="")
        else:
            return ReturnGroup(success=True, op=out)
    except OSError:
        return ReturnGroup(success=False, op="")


def build_config_diff_objs(commandname, command_result, current_modify_time):

    new_config_mon_obj = RunningConfigMon()
    new_config_mon_obj.commandname = commandname
    new_config_mon_obj.last_modify_time = current_modify_time
    new_config_mon_obj.result = command_result.op.decode()
    new_config_mon_obj.save()
    return new_config_mon_obj

def process_running_config_mon(cmd_output, svc_failed=False, **kwargs):
    try:
        ##conf_dict = configuration.get_config("config-diff")
        conf_dict = {
                        'running-config-diff':
                            {
                            'clag-params': 'clagctl params'
                            }
                     }
    except ValueError:
        m_configMon = RunningConfigMon.memmodel()
        for entry in m_configMon.query.filter(hostname=HOSTNAME):
            old_config_mon_ent = RunningConfigMon()
            old_config_mon_ent.copy(entry)
            old_config_mon_ent.delete()
        return ServiceStatus.FAIL

    fullstate = kwargs.get('fullstate')

    # grab current list of configMon objects
    m_configMon = RunningConfigMon.memmodel()
    old_configmon_objs = []
    full_state_arr = []
    deleteCommandist = []
    if m_configMon:
        old_configmon_objs = list(m_configMon.query.filter(hostname=HOSTNAME))
    if conf_dict.get('running-config-diff'):
        for commandname, command in list(conf_dict.get('running-config-diff').items()):
            result = runCommand(command)
            if result.success:
                mem_command_result = ""
                for old_entry in old_configmon_objs:
                    if old_entry.commandname == commandname:
                        mem_command_result = old_entry.result
                        break
                current_modify_time = time.time()

                if fullstate:
                    full_state_arr.append(build_config_diff_objs(commandname, result, current_modify_time))
                elif result.op != mem_command_result:
                    build_config_diff_objs(commandname, result, current_modify_time)

            else:
                deleteCommandist.append(commandname)
    for to_be_deleted_command in deleteCommandist:
        for old_obj in old_configmon_objs:
            if old_obj.commandname == to_be_deleted_command:
                old_configmon_ent = RunningConfigMon()
                old_configmon_ent.copy(old_obj)
                old_configmon_ent.delete()
                break


    if fullstate:
        save_fullstate(full_state_arr, RunningConfigMon.__name__)

    return ServiceStatus.OK
