# Copyright 2017, 2019, 2020 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt



import ujson as json

from netq_lib.orm.redisdb.models import (
    Temp,
    Fan,
    PSU,
    Power)
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate
from netq_lib.common.enums import Platform
local_sensor_dict = dict()
FAIL_TEMP = -273
FAIL_FAN = -1

dummy_sensor = [[{
    "s_power_in_input": 0.0,
    "s_voltage_in_max": 0.0,
    "s_voltage_out_input": 0.0,
    "s_power_out_max": 0.0,
    "s_power_in_max": 675.0,
    "s_voltage_in_input": 0.0,
    "s_voltage_out_max": 0.0,
    "s_adapter_name": "cumulus-vx-i2c-10-59",
    "s_voltage_in_min": 0.0,
    "s_voltage_out_min": 0.0,
    "s_power_out_input": 0.0
}, {
    "s_power_in_input": 72.375,
    "s_voltage_in_max": 0.0,
    "s_voltage_out_input": 11.957,
    "s_power_out_max": 0.0,
    "s_power_in_max": 675.0,
    "s_voltage_in_input": 214.0,
    "s_voltage_out_max": 0.0,
    "s_adapter_name": "cumulus-vx-i2c-10-58",
    "s_voltage_in_min": 0.0,
    "s_voltage_out_min": 0.0,
    "s_power_out_input": 57.5
}],
    [{
        "s_power_in_input": 3.0,
        "s_voltage_in_max": 0.0,
        "s_voltage_out_input": 230.0,
        "s_power_out_max": 0.0,
        "s_power_in_max": 175.0,
        "s_voltage_in_input": 200.0,
        "s_voltage_out_max": 0.0,
        "s_adapter_name": "cumulus40-vx-i2c-10-59",
        "s_voltage_in_min": 0.0,
        "s_voltage_out_min": 0.0,
        "s_power_out_input": 2.0
    }, {
        "s_power_in_input": 72.375,
        "s_voltage_in_max": 0.0,
        "s_voltage_out_input": 11.957,
        "s_power_out_max": 0.0,
        "s_power_in_max": 375.0,
        "s_voltage_in_input": 218.0,
        "s_voltage_out_max": 0.0,
        "s_adapter_name": "cumulus40-vx-i2c-10-58",
        "s_voltage_in_min": 0.0,
        "s_voltage_out_min": 0.0,
        "s_power_out_input": 57.5
    }]
]


def process_sensors(output, svc_failed=False, **kwargs):

    fullstate = kwargs.get('fullstate')
    if svc_failed:
        return ServiceStatus.FAIL
    else:
        try:
            s_list = json.loads(output)
        except ValueError:
            return ServiceStatus.FAIL

    temp_arr = []
    psu_arr = []
    fan_arr = []
    for s in s_list:
        sensor = None
        name = s['name']
        if not name:
            continue
        if s['type'] == 'temp':
            sensor = Temp()
            sensor.s_input = (float(s['input']) if s.get('input') is not None
                              else float(FAIL_TEMP))
            sensor.s_crit = int(s['crit']) if s.get('crit') is not None else FAIL_TEMP
            sensor.s_max = int(s['max']) if s.get('max') is not None else FAIL_TEMP
            sensor.s_min = int(s['min']) if s.get('min') is not None else FAIL_TEMP
            sensor.s_lcrit = int(s['lcrit']) if s.get('lcrit') is not None else FAIL_TEMP
            sensor.s_desc = s['description'].lower()
            # Naga: Temporarily comment below lines to have stream all the time
            #if local_sensor_dict.get(name):
            #    prev_val, prev_st = local_sensor_dict[name]
            #    if ((cur_st == prev_st) and
            #            (abs(sensor.s_input - prev_val) < 2.5)):
            #        continue
            #local_sensor_dict[name] = (sensor.s_input, cur_st)
        elif s['type'] == 'fan':
            sensor = Fan()
            sensor.s_input = int(s['input']) if s.get('input') is not None else FAIL_FAN
            sensor.s_max = int(s['max']) if s.get('max') is not None else FAIL_FAN
            sensor.s_min = int(s['min']) if s.get('min') is not None else FAIL_FAN
            sensor.s_desc = s['description'].lower()
            # Naga: Temporarily comment below lines to have stream all the time
            #if local_sensor_dict.get(name):
            #    prev_val, prev_st = local_sensor_dict[name]
            #    if ((cur_st == prev_st) and
            #            abs(sensor.s_input - prev_val) < s_max_val):
            #        continue
            #local_sensor_dict[name] = (sensor.s_input, cur_st)
        elif s['type'] == 'power':
            sensor = PSU()

        if sensor is not None:
            sensor.s_name = s['name'].lower()
            if not s['msg']:
                sensor.s_msg = ''
            else:
                msg = s['msg']

                msgs = msg.split('\n')
                m = []
                for msg in msgs:
                    if 'Unable to find driver path' in msg:
                        # ignore this msg
                        continue
                    if 'Unable to find pwm path' in msg:
                        # ignore this msg
                        continue
                    m.append(msg.split(':', 1)[1])
                if len(m):
                    sensor.s_msg = ','.join(m)
                else:
                    sensor.s_msg = ''
            sensor.s_state = s['state'].lower()
            sensor.s_prev_state = s['prev_state'].lower()

            if fullstate:
                if sensor.__class__.__name__ == "Temp":
                    temp_arr.append(sensor)
                elif sensor.__class__.__name__ == "Fan":
                    fan_arr.append(sensor)
                else:
                    psu_arr.append(sensor)
            else:
                sensor.save()
    if fullstate:
        save_fullstate(temp_arr, "Temp", False)
        save_fullstate(fan_arr, "Fan", False)
        save_fullstate(psu_arr, "PSU", False)

    return ServiceStatus.OK


def process_power(psu_output, svc_failed=False, **kwargs):
    fullstate = kwargs.get('fullstate')
    platform = kwargs.get('platform', '')
    random_platform = kwargs.get('random', '')
    power_arr = []
    required_fields = ['input', 'min', 'max']
    if Platform.VX in platform:
        if random_platform:
            random_index = int(random_platform) % 2
            random_power = dummy_sensor[random_index]
            for power_info in random_power:
                power_obj = Power()
                for k, v in list(power_info.items()):
                    setattr(power_obj, k, v)
                if not fullstate:
                    power_obj.save()
                power_arr.append(power_obj)
                
    if Platform.MELLANOX in platform:
        psu_output_list = [x.strip() for x in psu_output.split('\n')]
        index = 0
        psu_output_list.append('')
        while index < len(psu_output_list):
            start = index
            found_psu = False
            while psu_output_list[index] != '':
                if 'vin' in psu_output_list[index] or 'pin' in psu_output_list[index]:
                    found_psu = True
                    break
                index += 1
            index += 1
            if found_psu:
                index = start
            else:
                continue
            try:
                if found_psu:
                    power_obj = Power()
                    power_obj.s_adapter_name = psu_output_list[index]
                    index += 1
                    while psu_output_list[index] != '':
                        voltage_prefix = None
                        if psu_output_list[index].startswith("vin"):
                            voltage_prefix = psu_output_list[index + 1].split('_')[0]
                            voltage_attr = 's_voltage_in_'
                            index += 1
                        elif psu_output_list[index].startswith("vout"):
                            voltage_prefix = psu_output_list[index + 1].split('_')[0]
                            voltage_attr = 's_voltage_out_'
                            index += 1
                        if voltage_prefix:
                            while psu_output_list[index].startswith(voltage_prefix):
                                suffix = psu_output_list[index].split(':')
                                data = float(suffix[1].strip())
                                attr = suffix[0].split("_")[1]
                                if attr in required_fields:
                                    setattr(power_obj, voltage_attr + attr, data)
                                index += 1
                        power_prefix = None
                        if psu_output_list[index].startswith("pin"):
                            power_prefix = psu_output_list[index + 1].split('_')[0]
                            power_attr = 's_power_in_'
                            index += 1
                        elif psu_output_list[index].startswith("pout"):
                            power_prefix = psu_output_list[index + 1].split('_')[0]
                            power_attr = 's_power_out_'
                            index += 1
                        if power_prefix:
                            while psu_output_list[index].startswith(power_prefix):
                                suffix = psu_output_list[index].split(':')
                                data = float(suffix[1].strip())
                                attr = suffix[0].split("_")[1]
                                if attr in required_fields:
                                    setattr(power_obj, power_attr + attr, data)
                                index += 1
                        if not power_prefix and not voltage_prefix:
                            index += 1
                    if not fullstate:
                        power_obj.save()
                    power_arr.append(power_obj)
            except ValueError:
                power_arr = []
                break
    if fullstate:
        save_fullstate(power_arr, "Power", False)
    return ServiceStatus.OK
