# Copyright 2019 Cumulus Networks, Inc.  All Rights reserved.
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt


import ujson as json
from netq_lib.orm.redisdb.models import ForwardingResource
from netq_lib.orm.redisdb.models import ACLResource
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate


def process_tcam_resource(cmd_output, svc_failed=False, **kwargs):

    fullstate = kwargs.get('fullstate')
    try:
        json_out = json.loads(cmd_output)
    except ValueError:
        return ServiceStatus.FAIL

    forwarding_resource = ForwardingResource()
    acl_resource = ACLResource()
    forwarding_resource_attr = dir(forwarding_resource)
    acl_resource_attr = dir(acl_resource)

    for key, util in list(json_out.items()):
        util_json = {'max': util['max'], 'usage': util['count']}
        if key in forwarding_resource_attr:
            setattr(forwarding_resource, key, util_json)

        if key in acl_resource_attr:
            setattr(acl_resource, key, util_json)

    forwarding_resource.save()
    acl_resource.save()

    if fullstate is True:
        acl_resource_arr = []
        acl_resource_arr.append(acl_resource)
        save_fullstate(acl_resource_arr, ACLResource().__class__.__name__)

        fwd_res_arr = []
        fwd_res_arr.append(forwarding_resource)
        save_fullstate(fwd_res_arr, ForwardingResource().__class__.__name__)

    return ServiceStatus.OK
